@@ -219,8 +219,7 @@ int RXAFS_Lookup (struct rx_connection *,
 #define CM_ERROR_EXISTS			(CM_ERROR_BASE+11)
 #define CM_ERROR_CROSSDEVLINK		(CM_ERROR_BASE+12)
 #define CM_ERROR_BADOP			(CM_ERROR_BASE+13)
-#define CM_ERROR_BADSMB			(CM_ERROR_BASE+32)
-/* CM_ERROR_BADPASSWORD used to be here */
+#define CM_ERROR_BADPASSWORD            (CM_ERROR_BASE+14)
 #define CM_ERROR_NOTDIR			(CM_ERROR_BASE+15)
 #define CM_ERROR_ISDIR			(CM_ERROR_BASE+16)
 #define CM_ERROR_READONLY		(CM_ERROR_BASE+17)
@@ -236,9 +235,8 @@ int RXAFS_Lookup (struct rx_connection *,
 #define CM_ERROR_REMOTECONN		(CM_ERROR_BASE+27)
 #define CM_ERROR_ATSYS			(CM_ERROR_BASE+28)
 #define CM_ERROR_NOSUCHPATH		(CM_ERROR_BASE+29)
-
 #define CM_ERROR_CLOCKSKEW		(CM_ERROR_BASE+31)
-
+#define CM_ERROR_BADSMB			(CM_ERROR_BASE+32)
 #define CM_ERROR_ALLBUSY		(CM_ERROR_BASE+33)
 #define CM_ERROR_NOFILES		(CM_ERROR_BASE+34)
 #define CM_ERROR_PARTIALWRITE		(CM_ERROR_BASE+35)
@@ -247,5 +245,12 @@ int RXAFS_Lookup (struct rx_connection *,
 #define CM_ERROR_BUFFERTOOSMALL		(CM_ERROR_BASE+38)
 #define CM_ERROR_RENAME_IDENTICAL	(CM_ERROR_BASE+39)
 #define CM_ERROR_ALLOFFLINE             (CM_ERROR_BASE+40)
-
+#define CM_ERROR_AMBIGUOUS_FILENAME     (CM_ERROR_BASE+41)
+#define CM_ERROR_BADLOGONTYPE	        (CM_ERROR_BASE+42)
+#define CM_ERROR_GSSCONTINUE            (CM_ERROR_BASE+43)
+#define CM_ERROR_TIDIPC                 (CM_ERROR_BASE+44)
+#define CM_ERROR_TOO_MANY_SYMLINKS      (CM_ERROR_BASE+45)
+#define CM_ERROR_PATH_NOT_COVERED       (CM_ERROR_BASE+46)
+/* 47 and 48 are reserved for the byte range lock support */
+#define CM_ERROR_ALLDOWN                (CM_ERROR_BASE+49)
 #endif /*  __CM_H_ENV__ */
