@@ -38,7 +38,7 @@ extern long time;
 #define AFS_FAST_RELE(vp) (vp)->vrefCount--
 
 #undef gop_lookupname
-#define	gop_lookupname(fnamep,segflg,followlink,dirvpp,compvpp) \
+#define	gop_lookupname(fnamep,segflg,followlink,compvpp) \
 	lookupvp((fnamep), (followlink), (compvpp), &afs_osi_cred)
 
 #undef afs_suser
@@ -69,10 +69,6 @@ extern simple_lock_data afs_global_lock;
 			    simple_unlock((void *)&afs_global_lock);	\
 			} while(0)
 #define ISAFS_GLOCK()	lock_mine((void *)&afs_global_lock)
-
-#define AFS_RXGLOCK()
-#define AFS_RXGUNLOCK()
-#define ISAFS_RXGLOCK() 1
 #endif
 
 #endif /* _OSI_MACHDEP_H_ */
