@@ -18,7 +18,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_lookup.c,v 1.50.2.6 2005/01/31 03:49:15 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_lookup.c,v 1.50.2.10.2.2 2005/10/12 06:13:24 shadow Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
@@ -31,10 +31,6 @@ RCSID
 
 extern struct DirEntry *afs_dir_GetBlob();
 
-#ifdef AFS_LINUX22_ENV
-extern struct inode_operations afs_symlink_iops, afs_dir_iops;
-#endif
-
 
 afs_int32 afs_bkvolpref = 0;
 afs_int32 afs_bulkStatsDone;
@@ -436,7 +432,7 @@ Check_AtSys(register struct vcache *avc, const char *aname,
 	    struct sysname_info *state, struct vrequest *areq)
 {
     int num = 0;
-    char **sysnamelist[MAXSYSNAME];
+    char **sysnamelist[MAXNUMSYSNAMES];
 
     if (AFS_EQ_ATSYS(aname)) {
 	state->offset = 0;
@@ -457,7 +453,7 @@ Next_AtSys(register struct vcache *avc, struct vrequest *areq,
 	   struct sysname_info *state)
 {
     int num = afs_sysnamecount;
-    char **sysnamelist[MAXSYSNAME];
+    char **sysnamelist[MAXNUMSYSNAMES];
 
     if (state->index == -1)
 	return 0;		/* No list */
@@ -974,12 +970,8 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 	 * We only do this if the entry looks clear.
 	 */
 	afs_ProcessFS(tvcp, &statsp[i], areqp);
-#ifdef AFS_LINUX22_ENV
-	/* overwrite the ops if it's a directory or symlink. */
-	if (vType(tvcp) == VDIR)
-	    tvcp->v.v_op = &afs_dir_iops;
-	else if (vType(tvcp) == VLNK)
-	    tvcp->v.v_op = &afs_symlink_iops;
+#if defined(AFS_LINUX22_ENV)
+	afs_fill_inode(AFSTOV(tvcp), NULL);	/* reset inode operations */
 #endif
 
 	/* do some accounting for bulk stats: mark this entry as
@@ -1198,7 +1190,7 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
 
     /* Check for read access as well.  We need read access in order to
      * stat files, but not to stat subdirectories. */
-    if (!afs_AccessOK(adp, PRSFS_LOOKUP, &treq, CHECK_MODE_BITS))
+    if (!afs_AccessOK(adp, PRSFS_READ, &treq, CHECK_MODE_BITS))
 	no_read_access = 1;
 
     /* special case lookup of ".".  Can we check for it sooner in this code,
@@ -1241,7 +1233,7 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
 	}
 #ifdef AFS_LINUX22_ENV
 	if (tvc->mvstat == 2) {	/* we don't trust the dnlc for root vcaches */
-	    AFS_RELE(tvc);
+	    AFS_RELE(AFSTOV(tvc));
 	    *avcp = 0;
 	} else {
 	    code = 0;
