@@ -21,7 +21,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_write.c,v 1.36.2.3 2005/01/31 03:49:15 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_write.c,v 1.36.2.6 2005/10/03 02:55:33 shadow Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
@@ -159,16 +159,12 @@ afs_MemWrite(register struct vcache *avc, struct uio *auio, int aio,
      */
     avc->m.Date = startDate;
 
-#if	defined(AFS_HPUX_ENV) || defined(AFS_GFS_ENV)
+#if	defined(AFS_HPUX_ENV)
 #if	defined(AFS_HPUX101_ENV)
     if ((totalLength + filePos) >> 9 >
 	(p_rlimit(u.u_procp))[RLIMIT_FSIZE].rlim_cur) {
 #else
-#ifdef	AFS_HPUX_ENV
     if ((totalLength + filePos) >> 9 > u.u_rlimit[RLIMIT_FSIZE].rlim_cur) {
-#else
-    if (totalLength + filePos > u.u_rlimit[RLIMIT_FSIZE].rlim_cur) {
-#endif
 #endif
 	if (!noLock)
 	    ReleaseWriteLock(&avc->lock);
@@ -190,12 +186,13 @@ afs_MemWrite(register struct vcache *avc, struct uio *auio, int aio,
     avc->states |= CDirty;
     tvec = (struct iovec *)osi_AllocSmallSpace(sizeof(struct iovec));
     while (totalLength > 0) {
-	/* Read the cached info. If we call GetDCache while the cache
-	 * truncate daemon is running we risk overflowing the disk cache.
-	 * Instead we check for an existing cache slot. If we cannot
-	 * find an existing slot we wait for the cache to drain
-	 * before calling GetDCache.
+	/* 
+	 *  The following line is necessary because afs_GetDCache with
+	 *  flag == 4 expects the length field to be filled. It decides
+	 *  from this whether it's necessary to fetch data into the chunk
+	 *  before writing or not (when the whole chunk is overwritten!).
 	 */
+	len = totalLength;	/* write this amount by default */
 	if (noLock) {
 	    tdc = afs_FindDCache(avc, filePos);
 	    if (tdc)
@@ -327,11 +324,6 @@ afs_MemWrite(register struct vcache *avc, struct uio *auio, int aio,
     if (!noLock)
 	ReleaseWriteLock(&avc->lock);
     osi_FreeSmallSpace(tvec);
-#ifdef AFS_DEC_ENV
-    /* next, on GFS systems, we update g_size so that lseek's relative to EOF will
-     * work.  GFS is truly a poorly-designed interface!  */
-    afs_gfshack((struct gnode *)avc);
-#endif
     error = afs_CheckCode(error, &treq, 6);
     return error;
 }
@@ -410,16 +402,12 @@ afs_UFSWrite(register struct vcache *avc, struct uio *auio, int aio,
      */
     avc->m.Date = startDate;
 
-#if	defined(AFS_HPUX_ENV) || defined(AFS_GFS_ENV)
+#if	defined(AFS_HPUX_ENV)
 #if 	defined(AFS_HPUX101_ENV)
     if ((totalLength + filePos) >> 9 >
 	p_rlimit(u.u_procp)[RLIMIT_FSIZE].rlim_cur) {
 #else
-#ifdef	AFS_HPUX_ENV
     if ((totalLength + filePos) >> 9 > u.u_rlimit[RLIMIT_FSIZE].rlim_cur) {
-#else
-    if (totalLength + filePos > u.u_rlimit[RLIMIT_FSIZE].rlim_cur) {
-#endif
 #endif
 	if (!noLock)
 	    ReleaseWriteLock(&avc->lock);
@@ -573,7 +561,7 @@ afs_UFSWrite(register struct vcache *avc, struct uio *auio, int aio,
 	}
 #elif defined(AFS_LINUX20_ENV)
 	AFS_GUNLOCK();
-	code = osi_file_uio_rdwr(tfile, &tuio, UIO_WRITE);
+	code = osi_rdwr(tfile, &tuio, UIO_WRITE);
 	AFS_GLOCK();
 #elif defined(AFS_DARWIN_ENV)
 	AFS_GUNLOCK();
@@ -664,11 +652,6 @@ afs_UFSWrite(register struct vcache *avc, struct uio *auio, int aio,
     if (!noLock)
 	ReleaseWriteLock(&avc->lock);
     osi_FreeSmallSpace(tvec);
-#ifdef AFS_DEC_ENV
-    /* next, on GFS systems, we update g_size so that lseek's relative to EOF will
-     * work.  GFS is truly a poorly-designed interface!  */
-    afs_gfshack((struct gnode *)avc);
-#endif
 #ifndef	AFS_VM_RDWR_ENV
     /*
      * If write is implemented via VM, afs_fsync() is called from the high-level
@@ -770,11 +753,7 @@ afs_closex(register struct file *afd)
 	    if (flags)
 		HandleFlock(tvc, LOCK_UN, &treq, u.u_procp->p_pid,
 			    1 /*onlymine */ );
-#ifdef	AFS_DEC_ENV
-	    grele((struct gnode *)tvc);
-#else
 	    AFS_RELE(AFSTOV(tvc));
-#endif
 	    closeDone = 1;
 	}
     }
@@ -807,13 +786,9 @@ afs_close(OSI_VC_ARG(avc), aflags, lastclose,
      struct flid *flp;
 #endif
 #endif
-#elif	defined(AFS_SUN_ENV) || defined(AFS_SUN5_ENV)
-#ifdef	AFS_SUN5_ENV
+#elif defined(AFS_SUN5_ENV)
 afs_close(OSI_VC_ARG(avc), aflags, count, offset, acred)
      offset_t offset;
-#else
-afs_close(OSI_VC_ARG(avc), aflags, count, acred)
-#endif
      int count;
 #else
 afs_close(OSI_VC_ARG(avc), aflags, acred)
@@ -853,7 +828,7 @@ afs_close(OSI_VC_ARG(avc), aflags, acred)
 	afs_PutFakeStat(&fakestat);
 	return 0;
     }
-#elif	defined(AFS_SUN_ENV) || defined(AFS_SGI_ENV)
+#elif	defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
     if (count > 1) {
 	/* The vfs layer may call this repeatedly with higher "count"; only on the last close (i.e. count = 1) we should actually proceed with the close. */
 	afs_PutFakeStat(&fakestat);
@@ -880,7 +855,7 @@ afs_close(OSI_VC_ARG(avc), aflags, acred)
     afs_chkpgoob(&avc->v, btoc(avc->m.Length));
 #else /* AFS_SGI_ENV */
     if (avc->flockCount) {	/* Release Lock */
-#if	defined(AFS_OSF_ENV) || defined(AFS_SUN_ENV)
+#if	defined(AFS_OSF_ENV) 
 	HandleFlock(avc, LOCK_UN, &treq, u.u_procp->p_pid, 1 /*onlymine */ );
 #else
 	HandleFlock(avc, LOCK_UN, &treq, 0, 1 /*onlymine */ );
