@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/rx/rx_getaddr.c,v 1.15.2.1 2004/10/18 07:12:06 shadow Exp $");
+    ("$Header: /cvs/openafs/src/rx/rx_getaddr.c,v 1.15.2.7 2005/09/21 00:34:13 shadow Exp $");
 
 #ifndef AFS_DJGPP_ENV
 #ifndef KERNEL
@@ -33,7 +33,8 @@ RCSID
  * the pthreads for solaris requires the socket call to be mapped.
  */
 #include "rx.h"
-#endif
+#include "rx_globals.h"
+#endif /* AFS_NT40_ENV */
 #else /* KERNEL */
 #ifdef UKERNEL
 #include "rx/rx_kcommon.h"
@@ -169,7 +170,7 @@ rx_getAllAddr(afs_int32 buffer[], int maxSize)
     while (next < lim) {
 	ifm = (struct if_msghdr *)next;
 	if (ifm->ifm_type != RTM_IFINFO) {
-	    printf("out of sync parsing NET_RT_IFLIST\n");
+	    dpf(("out of sync parsing NET_RT_IFLIST\n"));
 	    free(buf);
 	    return 0;
 	}
@@ -204,8 +205,8 @@ rx_getAllAddr(afs_int32 buffer[], int maxSize)
 	    a = info.rti_info[RTAX_IFA];
 
 	    if (count >= maxSize)	/* no more space */
-		printf("Too many interfaces..ignoring 0x%x\n",
-		       a->sin_addr.s_addr);
+		dpf(("Too many interfaces..ignoring 0x%x\n",
+		       a->sin_addr.s_addr));
 	    else
 		buffer[count++] = a->sin_addr.s_addr;
 	    addrcount--;
@@ -253,7 +254,7 @@ rxi_getAllAddrMaskMtu(afs_int32 addrBuffer[], afs_int32 maskBuffer[],
     while (next < lim) {
 	ifm = (struct if_msghdr *)next;
 	if (ifm->ifm_type != RTM_IFINFO) {
-	    printf("out of sync parsing NET_RT_IFLIST\n");
+	    dpf(("out of sync parsing NET_RT_IFLIST\n"));
 	    free(buf);
 	    return 0;
 	}
@@ -284,26 +285,28 @@ rxi_getAllAddrMaskMtu(afs_int32 addrBuffer[], afs_int32 maskBuffer[],
 		continue;
 	    a = info.rti_info[RTAX_IFA];
 
-	    if (count >= maxSize) {	/* no more space */
-		printf("Too many interfaces..ignoring 0x%x\n",
-		       a->sin_addr.s_addr);
-	    } else {
-		struct ifreq ifr;
-
-		addrBuffer[count] = a->sin_addr.s_addr;
-		a = info.rti_info[RTAX_NETMASK];
-		if (a)
-		    maskBuffer[count] = a->sin_addr.s_addr;
-		else
-		    maskBuffer[count] = htonl(0xffffffff);
-		memset(&ifr, sizeof(ifr), 0);
-		ifr.ifr_addr.sa_family = AF_INET;
-		strncpy(ifr.ifr_name, sdl->sdl_data, sdl->sdl_nlen);
-		if (ioctl(s, SIOCGIFMTU, (caddr_t) & ifr) < 0)
-		    mtuBuffer[count] = htonl(1500);
-		else
-		    mtuBuffer[count] = htonl(ifr.ifr_mtu);
-		count++;
+	    if (a->sin_addr.s_addr != htonl(0x7f000001) ) {
+		if (count >= maxSize) {	/* no more space */
+		    dpf(("Too many interfaces..ignoring 0x%x\n",
+			   a->sin_addr.s_addr));
+		} else {
+		    struct ifreq ifr;
+		    
+		    addrBuffer[count] = a->sin_addr.s_addr;
+		    a = info.rti_info[RTAX_NETMASK];
+		    if (a)
+			maskBuffer[count] = a->sin_addr.s_addr;
+		    else
+			maskBuffer[count] = htonl(0xffffffff);
+		    memset(&ifr, sizeof(ifr), 0);
+		    ifr.ifr_addr.sa_family = AF_INET;
+		    strncpy(ifr.ifr_name, sdl->sdl_data, sdl->sdl_nlen);
+		    if (ioctl(s, SIOCGIFMTU, (caddr_t) & ifr) < 0)
+			mtuBuffer[count] = htonl(1500);
+		    else
+			mtuBuffer[count] = htonl(ifr.ifr_mtu);
+		    count++;
+		}
 	    }
 	    addrcount--;
 	    ifam = (struct ifa_msghdr *)((char *)ifam + ifam->ifam_msglen);
@@ -342,18 +345,22 @@ rx_getAllAddr_internal(afs_int32 buffer[], int maxSize, int loopbacks)
     for (cp = (char *)ifc.ifc_buf, cplim = ifc.ifc_buf + ifc.ifc_len;
 	 cp < cplim;
 #if defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
-	 cp += _SIZEOF_ADDR_IFREQ(*ifr))
+	 cp += _SIZEOF_ADDR_IFREQ(*ifr)
 #else
 #ifdef AFS_AIX51_ENV
-	 cp = cpnext)
+	 cp = cpnext
 #else
-	 cp += sizeof(ifr->ifr_name) + MAX(a->sin_len, sizeof(*a)))
+	 cp += sizeof(ifr->ifr_name) + MAX(a->sin_len, sizeof(*a))
 #endif
+#endif
+	) 
+#else
+    for (i = 0; i < len; ++i) 
 #endif
     {
+#if    defined(AFS_AIX41_ENV) || defined (AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
 	ifr = (struct ifreq *)cp;
 #else
-    for (i = 0; i < len; ++i) {
 	ifr = &ifs[i];
 #endif
 	a = (struct sockaddr_in *)&ifr->ifr_addr;
@@ -367,12 +374,16 @@ rx_getAllAddr_internal(afs_int32 buffer[], int maxSize, int loopbacks)
 	    continue;		/* ignore this address */
 	}
 	if (a->sin_addr.s_addr != 0) {
-	    if (!loopbacks && (ifr->ifr_flags & IFF_LOOPBACK)) {
-		continue;	/* skip aliased loopbacks as well. */
+            if (!loopbacks) {
+                if (a->sin_addr.s_addr == htonl(0x7f000001)) 
+		    continue;	/* skip loopback address as well. */
+            } else {
+                if (ifr->ifr_flags & IFF_LOOPBACK) 
+		    continue;	/* skip aliased loopbacks as well. */
 	    }
 	    if (count >= maxSize)	/* no more space */
-		printf("Too many interfaces..ignoring 0x%x\n",
-		       a->sin_addr.s_addr);
+		dpf(("Too many interfaces..ignoring 0x%x\n",
+		       a->sin_addr.s_addr));
 	    else
 		buffer[count++] = a->sin_addr.s_addr;
 	}
@@ -448,9 +459,12 @@ rxi_getAllAddrMaskMtu(afs_int32 addrBuffer[], afs_int32 maskBuffer[],
 		continue;	/* ignore this address */
 	    }
 
+            if (a->sin_addr.s_addr == htonl(0x7f000001) )
+                continue;   /* skip loopback address as well. */
+
 	    if (count >= maxSize) {	/* no more space */
-		printf("Too many interfaces..ignoring 0x%x\n",
-		       a->sin_addr.s_addr);
+		dpf(("Too many interfaces..ignoring 0x%x\n",
+		       a->sin_addr.s_addr));
 		continue;
 	    }
 
