@@ -37,7 +37,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/rxgen/rpc_main.c,v 1.20 2003/07/15 23:16:41 shadow Exp $");
+    ("$Header: /cvs/openafs/src/rxgen/rpc_main.c,v 1.21.2.1 2005/04/03 18:15:51 shadow Exp $");
 
 #include <limits.h>
 #include <stdio.h>
@@ -476,9 +476,6 @@ h_output(char *infile, char *define, int extend, char *outfile, int append)
     f_print(fout, "#include \"h/socket.h\"\n");
     f_print(fout, "#endif\n");
     f_print(fout, "#ifndef	DTYPE_SOCKET  /* XXXXX */\n");
-    f_print(fout, "#ifdef AFS_DEC_ENV\n");
-    f_print(fout, "#include \"h/smp_lock.h\"\n");
-    f_print(fout, "#endif\n");
     f_print(fout, "#ifndef AFS_LINUX22_ENV\n");
     f_print(fout, "#include \"h/file.h\"\n");
     f_print(fout, "#endif\n");
@@ -785,7 +782,7 @@ parseargs(int argc, char *argv[], struct commandline *cmd)
     int i;
     int j;
     char c;
-    char flag[(1 << 8 * sizeof(char))];
+    char flag[(1 << (8 * sizeof(char)))];
     int nflags;
 
     cmdname = argv[0];
