@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/viced/host.c,v 1.57.2.1 2004/10/18 07:12:22 shadow Exp $");
+    ("$Header: /cvs/openafs/src/viced/host.c,v 1.57.2.5 2005/07/28 20:52:21 shadow Exp $");
 
 #include <stdio.h>
 #include <errno.h>
@@ -947,6 +947,7 @@ h_GetHost_r(struct rx_connection *tcon)
     afs_int32 hport;
     char hoststr[16], hoststr2[16];
     Capabilities caps;
+    struct rx_connection *cb_conn = NULL;
 
     caps.Capabilities_val = NULL;
 
@@ -979,11 +980,15 @@ h_GetHost_r(struct rx_connection *tcon)
 	    goto retry;
 	}
 	host->hostFlags &= ~ALTADDR;
+	cb_conn = host->callback_rxcon;
+	rx_GetConnection(cb_conn);
 	H_UNLOCK;
 	code =
-	    RXAFSCB_TellMeAboutYourself(host->callback_rxcon, &interf, &caps);
+	    RXAFSCB_TellMeAboutYourself(cb_conn, &interf, &caps);
 	if (code == RXGEN_OPCODE)
-	    code = RXAFSCB_WhoAreYou(host->callback_rxcon, &interf);
+	    code = RXAFSCB_WhoAreYou(cb_conn, &interf);
+	rx_PutConnection(cb_conn);
+	cb_conn=NULL;
 	H_LOCK;
 	if (code == RXGEN_OPCODE) {
 	    identP = (struct Identity *)malloc(sizeof(struct Identity));
@@ -1098,12 +1103,15 @@ h_GetHost_r(struct rx_connection *tcon)
 	h_gethostcps_r(host, FT_ApproxTime());
 	if (!(host->Console & 1)) {
 	    int pident = 0;
+	    cb_conn = host->callback_rxcon;
+	    rx_GetConnection(cb_conn);
 	    H_UNLOCK;
 	    code =
-		RXAFSCB_TellMeAboutYourself(host->callback_rxcon, &interf,
-					    &caps);
+		RXAFSCB_TellMeAboutYourself(cb_conn, &interf, &caps);
 	    if (code == RXGEN_OPCODE)
-		code = RXAFSCB_WhoAreYou(host->callback_rxcon, &interf);
+		code = RXAFSCB_WhoAreYou(cb_conn, &interf);
+	    rx_PutConnection(cb_conn);
+	    cb_conn=NULL;
 	    H_LOCK;
 	    if (code == RXGEN_OPCODE) {
 		if (!identP)
@@ -1146,36 +1154,75 @@ h_GetHost_r(struct rx_connection *tcon)
 			 ntohs(host->port)));
 	    }
 	    if (code == 0 && !identP->valid) {
+	        cb_conn = host->callback_rxcon;
+	        rx_GetConnection(cb_conn);
 		H_UNLOCK;
-		code = RXAFSCB_InitCallBackState(host->callback_rxcon);
+		code = RXAFSCB_InitCallBackState(cb_conn);
+	        rx_PutConnection(cb_conn);
+	        cb_conn=NULL;
 		H_LOCK;
 	    } else if (code == 0) {
 		oldHost = h_LookupUuid_r(&identP->uuid);
+                if (oldHost) {
+                    int probefail = 0;
+
+		    if (!(held = h_Held_r(oldHost)))
+			h_Hold_r(oldHost);
+		    h_Lock_r(oldHost);
+
+                    if (oldHost->interface) {
+			afsUUID uuid = oldHost->interface->uuid;
+                        cb_conn = host->callback_rxcon;
+                        rx_GetConnection(cb_conn);
+			H_UNLOCK;
+			code = RXAFSCB_ProbeUuid(cb_conn, &uuid);
+                        rx_PutConnection(cb_conn);
+                        cb_conn=NULL;
+			H_LOCK;
+			if (code && MultiProbeAlternateAddress_r(oldHost)) {
+                            probefail = 1;
+                        }
+                    } else {
+                        probefail = 1;
+                    }
+
+                    if (probefail) {
+                        /* The old host is either does not have a Uuid,
+                         * is not responding to Probes, 
+                         * or does not have a matching Uuid. 
+                         * Delete it! */
+                        oldHost->hostFlags |= HOSTDELETED;
+                        h_Unlock_r(oldHost);
+                        h_Release_r(oldHost);
+                        oldHost = NULL;
+                    }
+                }
 		if (oldHost) {
 		    /* This is a new address for an existing host. Update
 		     * the list of interfaces for the existing host and
 		     * delete the host structure we just allocated. */
-		    if (!(held = h_Held_r(oldHost)))
-			h_Hold_r(oldHost);
-		    h_Lock_r(oldHost);
 		    ViceLog(25,
 			    ("CB: new addr %s:%d for old host %s:%d\n",
 			     afs_inet_ntoa_r(host->host, hoststr),
 			     ntohs(host->port), afs_inet_ntoa_r(oldHost->host,
 								hoststr2),
 			     ntohs(oldHost->port)));
+		    addInterfaceAddr_r(oldHost, haddr);
 		    host->hostFlags |= HOSTDELETED;
 		    h_Unlock_r(host);
 		    h_Release_r(host);
 		    host = oldHost;
-		    addInterfaceAddr_r(host, haddr);
 		} else {
 		    /* This really is a new host */
 		    hashInsertUuid_r(&identP->uuid, host);
+		    cb_conn = host->callback_rxcon;
+		    rx_GetConnection(cb_conn);		
 		    H_UNLOCK;
 		    code =
-			RXAFSCB_InitCallBackState3(host->callback_rxcon,
+			RXAFSCB_InitCallBackState3(cb_conn,
 						   &FS_HostUUID);
+		    rx_PutConnection(cb_conn);
+		    cb_conn=NULL;
 		    H_LOCK;
 		    if (code == 0) {
 			ViceLog(25,
@@ -1202,7 +1249,7 @@ h_GetHost_r(struct rx_connection *tcon)
 	    host->hostFlags |= HERRORTRANS;
 	else
 	    host->hostFlags &= ~(HERRORTRANS);
-	host->hostFlags |= ALTADDR;	/* host structure iniatilisation complete */
+	host->hostFlags |= ALTADDR;	/* host structure initialization complete */
 	h_Unlock_r(host);
     }
     if (caps.Capabilities_val)
@@ -1312,7 +1359,7 @@ MapName_r(char *aname, char *acell, afs_int32 * aval)
 /*MapName*/
 
 
-/* NOTE: this returns the client with a Shared lock */
+/* NOTE: this returns the client with a Write lock */
 struct client *
 h_ID2Client(afs_int32 vid)
 {
@@ -1327,7 +1374,7 @@ h_ID2Client(afs_int32 vid)
 	    if (!client->deleted && client->ViceId == vid) {
 		client->refCount++;
 		H_UNLOCK;
-		ObtainSharedLock(&client->lock);
+		ObtainWriteLock(&client->lock);
 		H_LOCK;
 		client->refCount--;
 		H_UNLOCK;
@@ -1337,7 +1384,7 @@ h_ID2Client(afs_int32 vid)
     }
 
     H_UNLOCK;
-    return 0;
+    return NULL;
 }
 
 /*
@@ -1436,6 +1483,7 @@ h_FindClient_r(struct rx_connection *tcon)
     if (!client) {
 	host = h_GetHost_r(tcon);	/* Returns it h_Held */
 
+    retryfirstclient:
 	/* First try to find the client structure */
 	for (client = host->FirstClient; client; client = client->next) {
 	    if (!client->deleted && (client->sid == rxr_CidOf(tcon))
@@ -1469,6 +1517,15 @@ h_FindClient_r(struct rx_connection *tcon)
 
 	/* Still no client structure - get one */
 	if (!client) {
+	    h_Lock_r(host);
+	    /* Retry to find the client structure */
+	    for (client = host->FirstClient; client; client = client->next) {
+		if (!client->deleted && (client->sid == rxr_CidOf(tcon))
+		    && (client->VenusEpoch == rxr_GetEpoch(tcon))) {
+		    h_Unlock_r(host);
+		    goto retryfirstclient;
+		}
+	    }
 	    client = GetCE();
 	    ObtainWriteLock(&client->lock);
 	    client->refCount = 1;
@@ -1485,6 +1542,7 @@ h_FindClient_r(struct rx_connection *tcon)
 	    client->VenusEpoch = rxr_GetEpoch(tcon);
 	    client->CPS.prlist_val = 0;
 	    client->CPS.prlist_len = 0;
+	    h_Unlock_r(host);
 	    CurrentConnections++;	/* increment number of connections */
 	}
     }
@@ -2043,6 +2101,7 @@ int
 CheckHost(register struct host *host, int held)
 {
     register struct client *client;
+    struct rx_connection *cb_conn = NULL;
     int code;
 
     /* Host is held by h_Enumerate */
@@ -2055,6 +2114,8 @@ CheckHost(register struct host *host, int held)
     }
     if (host->LastCall < checktime) {
 	h_Lock_r(host);
+	cb_conn = host->callback_rxcon;
+	rx_GetConnection(cb_conn);
 	if (!(host->hostFlags & HOSTDELETED)) {
 	    if (host->LastCall < clientdeletetime) {
 		host->hostFlags |= HOSTDELETED;
@@ -2063,13 +2124,13 @@ CheckHost(register struct host *host, int held)
 		    if (host->interface) {
 			H_UNLOCK;
 			code =
-			    RXAFSCB_InitCallBackState3(host->callback_rxcon,
+			    RXAFSCB_InitCallBackState3(cb_conn,
 						       &FS_HostUUID);
 			H_LOCK;
 		    } else {
 			H_UNLOCK;
 			code =
-			    RXAFSCB_InitCallBackState(host->callback_rxcon);
+			    RXAFSCB_InitCallBackState(cb_conn);
 			H_LOCK;
 		    }
 		    host->hostFlags |= ALTADDR;	/* alternate addresses valid */
@@ -2093,7 +2154,7 @@ CheckHost(register struct host *host, int held)
 		    if (host->interface) {
 			afsUUID uuid = host->interface->uuid;
 			H_UNLOCK;
-			code = RXAFSCB_ProbeUuid(host->callback_rxcon, &uuid);
+			code = RXAFSCB_ProbeUuid(cb_conn, &uuid);
 			H_LOCK;
 			if (code) {
 			    if (MultiProbeAlternateAddress_r(host)) {
@@ -2107,7 +2168,7 @@ CheckHost(register struct host *host, int held)
 			}
 		    } else {
 			H_UNLOCK;
-			code = RXAFSCB_Probe(host->callback_rxcon);
+			code = RXAFSCB_Probe(cb_conn);
 			H_LOCK;
 			if (code) {
 			    char hoststr[16];
@@ -2121,6 +2182,10 @@ CheckHost(register struct host *host, int held)
 		}
 	    }
 	}
+	H_UNLOCK;
+	rx_PutConnection(cb_conn);
+	cb_conn=NULL;
+	H_LOCK;
 	h_Unlock_r(host);
     }
     H_UNLOCK;
