@@ -13,7 +13,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/vol/namei_ops.c,v 1.21.2.1 2004/08/25 07:14:19 shadow Exp $");
+    ("$Header: /cvs/openafs/src/vol/namei_ops.c,v 1.21.2.3 2004/11/09 17:16:40 shadow Exp $");
 
 #ifdef AFS_NAMEI_ENV
 #include <stdio.h>
@@ -229,7 +229,8 @@ namei_HandleToName(namei_t * name, IHandle_t * ih)
  * name space.
  */
 #define VICE_README "These files and directories are a part of the AFS \
-namespace. Modifying them\nin any way will result in loss of AFS data.\n"
+namespace. Modifying them\nin any way will result in loss of AFS data,\n\
+ownership and permissions included.\n"
 int
 namei_ViceREADME(char *partition)
 {
@@ -1041,8 +1042,8 @@ static int namei_ListAFSSubDirs(IHandle_t * dirIH,
 						  struct ViceInodeInfo *,
 						  char *, char *), FILE * fp,
 				int (*judgeFun) (struct ViceInodeInfo *,
-						 int vid),
-				int singleVolumeNumber);
+						 int vid, void *),
+				int singleVolumeNumber, void *rock);
 
 
 /* WriteInodeInfo
@@ -1093,8 +1094,9 @@ VerifyDirPerms(char *path)
  */
 int
 ListViceInodes(char *devname, char *mountedOn, char *resultFile,
-	       int (*judgeInode) (struct ViceInodeInfo * info, int vid),
-	       int singleVolumeNumber, int *forcep, int forceR, char *wpath)
+	       int (*judgeInode) (struct ViceInodeInfo * info, int vid, void *rock),
+	       int singleVolumeNumber, int *forcep, int forceR, char *wpath, 
+	       void *rock)
 {
     FILE *fp = (FILE *) - 1;
     int ninodes;
@@ -1114,7 +1116,7 @@ ListViceInodes(char *devname, char *mountedOn, char *resultFile,
 
     ninodes =
 	namei_ListAFSFiles(mountedOn, WriteInodeInfo, fp, judgeInode,
-			   singleVolumeNumber);
+			   singleVolumeNumber, rock);
 
     if (!resultFile)
 	return ninodes;
@@ -1167,8 +1169,8 @@ int
 namei_ListAFSFiles(char *dev,
 		   int (*writeFun) (FILE *, struct ViceInodeInfo *, char *,
 				    char *), FILE * fp,
-		   int (*judgeFun) (struct ViceInodeInfo *, int),
-		   int singleVolumeNumber)
+		   int (*judgeFun) (struct ViceInodeInfo *, int, void *),
+		   int singleVolumeNumber, void *rock)
 {
     IHandle_t ih;
     namei_t name;
@@ -1188,7 +1190,7 @@ namei_ListAFSFiles(char *dev,
 	namei_HandleToVolDir(&name, &ih);
 	ninodes =
 	    namei_ListAFSSubDirs(&ih, writeFun, fp, judgeFun,
-				 singleVolumeNumber);
+				 singleVolumeNumber, rock);
 	if (ninodes < 0)
 	    return ninodes;
     } else {
@@ -1212,7 +1214,7 @@ namei_ListAFSFiles(char *dev,
 		    if (!DecodeVolumeName(dp2->d_name, &ih.ih_vid)) {
 			ninodes +=
 			    namei_ListAFSSubDirs(&ih, writeFun, fp, judgeFun,
-						 0);
+						 0, rock);
 		    }
 		}
 		closedir(dirp2);
@@ -1239,8 +1241,8 @@ static int
 namei_ListAFSSubDirs(IHandle_t * dirIH,
 		     int (*writeFun) (FILE *, struct ViceInodeInfo *, char *,
 				      char *), FILE * fp,
-		     int (*judgeFun) (struct ViceInodeInfo *, int),
-		     int singleVolumeNumber)
+		     int (*judgeFun) (struct ViceInodeInfo *, int, void *),
+		     int singleVolumeNumber, void *rock)
 {
     IHandle_t myIH = *dirIH;
     namei_t name;
@@ -1283,7 +1285,7 @@ namei_ListAFSSubDirs(IHandle_t * dirIH,
 		info.linkCount =
 		    namei_GetLinkCount(&linkHandle, (Inode) 0, 0);
 	    }
-	    if (judgeFun && !(*judgeFun) (&info, singleVolumeNumber))
+	    if (judgeFun && !(*judgeFun) (&info, singleVolumeNumber, rock))
 		continue;
 
 	    if ((*writeFun) (fp, &info, path1, dp1->d_name) < 0) {
@@ -1345,7 +1347,7 @@ namei_ListAFSSubDirs(IHandle_t * dirIH,
 				continue;
 			    }
 			    if (judgeFun
-				&& !(*judgeFun) (&info, singleVolumeNumber))
+				&& !(*judgeFun) (&info, singleVolumeNumber, rock))
 				continue;
 
 			    if ((*writeFun) (fp, &info, path3, dp3->d_name) <
